/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.GQuery;
import com.vaadin.terminal.gwt.client.UIDL;

public class Filtering extends TableComponent {

	private String mainId = "#v-dtable";
	private String frozenBodyId = "table#dt-frozen-body-table";
	private String bodyId = "table#dt-body-table";
	private String filteringHtml = new String();

	private Map<Integer, List<Object>> columnsFiltering = new HashMap<Integer, List<Object>>();

	public Filtering(VDTable dTable) {
		super(dTable);
	}

	private void setFiltering(String[] filterings) {
		if (filterings != null && filterings.length > 0) {
			Map<Integer, List<Object>> newFilters = new HashMap<Integer, List<Object>>();
			for (int filter = 0; filter < filterings.length; filter += 3) {
				int column = Integer.parseInt(filterings[filter]);
				List<Object> filters = newFilters.get(column);
				if (filters == null) {
					filters = new ArrayList<Object>();
				}
				filters.add(new String[] { filterings[filter + 1],
						filterings[filter + 2] });
				newFilters.put(column, filters);
			}
			if (!newFilters.isEmpty()) {
				columnsFiltering = new HashMap<Integer, List<Object>>(
						newFilters);
			}
		}
	}

	private void setFilteringHtml(String filterHtml) {
		this.filteringHtml = filterHtml;
	}

	private void showFilteringHtml() {
		$(mainId + " div#dt-filter-info").remove();
		if (getDTable().config().isEnabledFilterInfo()
				&& !filteringHtml.isEmpty()) {
			filteringHtml = "<b><u>"
					+ getDTable().translation().get(Translation.FILTERS)
					+ "</u></b> " + filteringHtml;

			GQuery mainContainerWrapper = $(mainId).find(
					"div#dt-main-container-wrapper");
			GQuery tableTools = $(mainId).find(
					"div#dt-table-tools" + getDTable().getId());
			if (getDTable().config().isFilterInfoOnTop()) {
				if (getDTable().tools().isVisible()
						&& getDTable().tools().isOnTop()) {
					tableTools.before($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filteringHtml.toString()));
				} else {
					mainContainerWrapper.before($("<div/>")
							.id("dt-filter-info").addClass("dt-filter-info")
							.html(filteringHtml.toString()));
				}
			} else if (getDTable().config().isFilterInfoOnBottom()) {
				if (getDTable().tools().isVisible()
						&& getDTable().tools().isOnBottom()) {
					tableTools.after($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filteringHtml.toString()));
				} else {
					mainContainerWrapper.after($("<div/>").id("dt-filter-info")
							.addClass("dt-filter-info")
							.html(filteringHtml.toString()));
				}
			}
		}
		filteringHtml = new String();
	}

	public void unmarkFilteringColumn(int column) {
		String filterId = "td [id$=':" + column + "']";
		$(getDTable().isFrozen(column) ? frozenBodyId : bodyId).children("tr")
				.children(filterId).removeClass("filter");
	}

	public boolean isFilteringEnabled(int column) {
		return (getDTable().getAllRows() > 1 && (!getDTable().isFrozen(column) ? getDTable()
				.config().isEnabledFilteredTable() : getDTable().config()
				.isEnabledFilteredFrozenTable()));
	}

	public boolean isFiltered(int column) {
		return (columnsFiltering.containsKey(column));
	}

	public boolean filterOperatorExists(int column, String operator) {
		if (isFiltered(column)) {
			List<Object> filters = columnsFiltering.get(column);
			for (Object filter : filters) {
				if (((String[]) filter)[0].equals(operator)) {
					return true;
				}
			}
		}
		return false;
	}

	public boolean filteringExists(int column, String filter) {
		if (isFiltered(column)) {
			List<Object> filters = columnsFiltering.get(column);
			for (Object tempFilter : filters) {
				if (((String[]) tempFilter)[1].equals(filter)) {
					return true;
				}
			}
		}
		return false;
	}

	public void newFiltering(int column, int row, String operator,
			String filter, String cellValue) {
		if (operator != null && !operator.isEmpty()) {
			getDTable()
					.sendMessage(
							"newFiltering",
							(Integer.toString(column) + "_::_"
									+ Integer.toString(row) + "_::_" + operator
									+ "_::_" + filter + "_::_" + cellValue),
							true);
		}
	}

	public void removeFiltering(int column) {
		if (isFiltered(column)
				|| getDTable().searchColumns().isSearched(column)) {
			unmarkFilteringColumn(column);
			getDTable().searchColumns().emptySearchingColumn(column);
			getDTable().sendMessage("removeFiltering",
					Integer.toString(column), true);
		}
	}

	public void markFilteringColumns() {
		if (((getDTable().config().isEnabledFilteredFrozenTable() || getDTable()
				.config().isEnabledFilteredTable()) && !columnsFiltering
				.isEmpty())
				|| (getDTable().config().isEnabledSearchColumns() && !getDTable()
						.searchColumns().isEmpty())) {
			List<Integer> columns = new ArrayList<Integer>(
					columnsFiltering.keySet());
			columns.addAll(getDTable().searchColumns().getSearchingColumns());
			Set<Integer> colSet = new HashSet<Integer>(columns);
			columns = new ArrayList<Integer>(colSet);
			Collections.sort(columns);

			for (Integer column : columns) {
				String filterId = "td [id$=':" + column + "']";
				$(getDTable().isFrozen(column) ? frozenBodyId : bodyId)
						.children("tr").children(filterId).addClass("filter");
			}
		}
	}

	@Override
	public void setIds() {
		if (!mainId.endsWith("" + getDTable().getId())) {
			mainId += getDTable().getId();
			frozenBodyId += getDTable().getId();
			bodyId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		columnsFiltering.clear();
		if (uidl.hasAttribute("newFiltering")) {
			setFiltering(uidl.getStringArrayAttribute("newFiltering"));
		}

		if (uidl.hasAttribute("filteringHtml")) {
			setFilteringHtml(uidl.getStringAttribute("filteringHtml"));
		}
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
		$(mainId + " div#dt-filter-info").remove();
	}

	public void apply() {
		if (((getDTable().config().isEnabledFilteredFrozenTable() || getDTable()
				.config().isEnabledFilteredTable()) && !columnsFiltering
				.isEmpty())
				|| (getDTable().config().isEnabledSearchColumns() && !getDTable()
						.searchColumns().isEmpty())) {
			showFilteringHtml();
		} else {
			unload();
		}
	}

	@Override
	public void enableEventsHandler() {
	}

	@Override
	public void disableEventsHandler() {
	}

}
