/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.table;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.client.components.Tooltip;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;
import org.dussan.vaadin.dtable.client.components.events.TableEvent;
import org.dussan.vaadin.dtable.client.ui.VDTable;

import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.query.client.css.RGBColor;
import com.google.gwt.query.client.css.TakesCssValue.CssSetter;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.vaadin.terminal.gwt.client.BrowserInfo;
import com.vaadin.terminal.gwt.client.UIDL;

public class TableHeader extends TableComponent {

	private String frozenHeaderId = "table#dt-frozen-header-table";
	private String headerId = "table#dt-header-table";

	private boolean show = false;
	private List<Integer> columns = new ArrayList<Integer>();
	private Map<Integer, String> headers = new HashMap<Integer, String>();
	private Map<Integer, String> tooltips = new HashMap<Integer, String>();
	private Map<Integer, Integer> columnsAlignment = new HashMap<Integer, Integer>();
	private Map<Integer, Integer> columnsWidth = new HashMap<Integer, Integer>();
	private Tooltip tooltip = null;

	private GQuery frozenCellMouseOverEventHandler = null;
	private GQuery frozenCellClickEventHandler = null;
	private GQuery frozenCellContextMenuEventHandler = null;
	private GQuery cellMouseOverEventHandler = null;
	private GQuery cellClickEventHandler = null;
	private GQuery cellContextMenuEventHandler = null;

	public TableHeader(VDTable dTable) {
		super(dTable);
		tooltip = new Tooltip();
	}

	public Tooltip tooltip() {
		return tooltip;
	}

	private void setColumns(int[] columns) {
		this.columns.clear();
		for (int column : columns) {
			this.columns.add(column);
		}
	}

	private void setColumnsWidth(int[] widths) {
		columnsWidth.clear();
		for (int column = 0; column < widths.length; column++) {
			columnsWidth.put(columns.get(column), widths[column]);
		}
	}

	private void setColumnsWidth(Map<Integer, Integer> columnsWidth) {
		if (columnsWidth != null && columnsWidth.size() > 0) {
			this.columnsWidth = new HashMap<Integer, Integer>(columnsWidth);
		}
	}

	public String getHeader(int column) {
		return headers.get(column);
	}

	private void setHeaders(String[] headers) {
		if (headers != null && headers.length > 0) {
			this.headers.clear();
			for (int column = 0; column < headers.length; column++) {
				this.headers.put(columns.get(column), headers[column]);
			}
		}
	}

	public String getTooltip(int column) {
		return tooltips.get(column);
	}

	private void setTooltips(String[] tooltips) {
		if (tooltips != null && tooltips.length > 0) {
			this.tooltips.clear();
			for (int column = 0; column < tooltips.length; column++) {
				this.tooltips.put(columns.get(column), tooltips[column]);
			}
		}
	}

	private void setColumnsAlignment(int[] columnsAlignment) {
		if (columnsAlignment != null && columnsAlignment.length > 0) {
			this.columnsAlignment.clear();
			for (int column = 0; column < columnsAlignment.length; column++) {
				this.columnsAlignment.put(columns.get(column),
						columnsAlignment[column]);
			}
		}
	}

	private void emptyHeader() {
		$(headerId).empty().append($("<tr/>").id(getDTable().getId() + "t"));
		if (getDTable().config().isEnabledFrozenTable()) {
			int width = !getDTable().isFrozenTableVisible() ? 0 : getDTable()
					.getFrozenTableWidth();
			$(frozenHeaderId).empty().append(
					$("<tr/>").id(getDTable().getId() + "ft"));
			$(frozenHeaderId).parent().parent()
					.css(CSS.WIDTH.with((Length.px(width))));
		} else {
			$(frozenHeaderId).remove();
		}
	}

	private void fillHeader() {
		int controledWidth = -$(headerId).parent().scrollLeft()
				+ getDTable().getLeft();
		int tableWidth = BrowserInfo.get().getScreenWidth();
		String clazz = getDTable().config().isEnabledWordWrapInHeaderCells() ? "normal"
				: "nowrap";
		String id = getDTable().getId() + "h:0:";
		CssSetter height = CSS.HEIGHT.with(Length.px(getDTable().config()
				.getHeaderRowHeight()
				- getDTable().config().getHeaderCellMarginTop()
				- getDTable().config().getHeaderCellMarginBottom() - 1));
		CssSetter lineHeight = CSS.LINE_HEIGHT.with(Length.px(getDTable()
				.config().getHeaderLineHeight()));
		CssSetter margins = CSS.MARGIN.with(
				Length.px(getDTable().config().getHeaderCellMarginTop()),
				Length.px(getDTable().config().getHeaderCellMarginRight()),
				Length.px(getDTable().config().getHeaderCellMarginBottom()),
				Length.px(getDTable().config().getHeaderCellMarginLeft()));

		for (int column : columns) {
			CssSetter align = CSS.TEXT_ALIGN
					.with(VDTable.COLUMN_ALIGN[columnsAlignment.get(column)]);
			CssSetter width = CSS.WIDTH.with(Length.px(columnsWidth.get(column)
					- getDTable().getBrowserOffset()));
			$("<div/>")
					.id(id + column + ":1")
					.css(height)
					.addClass(clazz)
					.css(margins)
					.append($("<span/>").id(id + column + ":2").css(lineHeight)
							.text(headers.get(column)))
					.appendTo(
							$("<td/>").id(id + column).css(align).css(width)
									.appendTo($(headerId).children("tr")));

			// only display columns which are in screen area
			if (!getDTable().isFrozen(column)
					&& !(getDTable().config().isEnabledAllHeaderColumns() || getDTable()
							.config().isEnabledAllColumns())) {
				controledWidth += columnsWidth.get(column);
				if (controledWidth > tableWidth) {
					getDTable().setAllColumnsVisible(false);
					break;
				}
			}
		}
	}

	private void setFrozenHeader() {
		if (getDTable().config().isEnabledFrozenTable()) {
			String id = "td [id='" + getDTable().getId() + "h:0:_']";
			if (columns != null) {
				for (int column : columns) {
					if (getDTable().isFrozen(column)) {
						GQuery cell = $(headerId).children("tr").children(
								id.replace("_", "" + column));
						$(frozenHeaderId).children("tr").append(cell.clone());
						cell.remove();
					}
				}
			}
			removeRightBorderFromLastColumn(frozenHeaderId);
		}
	}

	private void removeExpandedCells() {
		if (getDTable().config().isEnabledFrozenTable()) {
			$(frozenHeaderId).children("tr").eq(0).find("div#clone").parent()
					.children("div.normal", "div.nowrap").css(CSS.DISPLAY, "")
					.parent().children("div#clone").remove();
		}
		$(headerId).children("tr").eq(0).find("div#clone").parent()
				.children("div.normal", "div.nowrap").css(CSS.DISPLAY, "")
				.parent().children("div#clone").remove();
	}

	private void removeVerticalLines() {
		if (!getDTable().config().isEnabledHeaderVerticalLines()) {
			$(headerId).children("tr").children("td")
					.css(CSS.BORDER_LEFT_COLOR.with(RGBColor.TRANSPARENT));
			$(headerId).children("tr").children("td")
					.css(CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
		}
	}

	private void removeRightBorderFromLastColumn(String header) {
		$(header).children("tr").children("td").last()
				.css(CSS.BORDER_RIGHT_COLOR.with(RGBColor.TRANSPARENT));
	}

	private void activateFrozenCellMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateFrozenCellMouseOverEventHandler(false);
			frozenCellMouseOverEventHandler = $(
					"table#dt-frozen-header-table" + getDTable().getId())
					.delegate("td", "mouseover", new Function() {
						public boolean f(Event e) {
							TableEvent.cellMouseOver(getDTable(), $(e), null);
							if (getDTable().config().isEnabledTooltip()
									&& !getDTable().menu().isShowing()
									&& "0".equals($(e).id().split(":")[1])) {
								int column = Integer.parseInt($(e).id().split(
										":")[2]);
								tooltip.setTo(e, getTooltip(column),
										getDTable().config()
												.isEnabledWordWrapInTooltip(),
										getDTable().config()
												.getTooltipMinimumWidth(),
										getDTable().config()
												.getTooltipMaximumWidth(),
										getDTable().config()
												.getTooltipPosition());
								tooltip.show();
							}
							return true;
						}
					}).delegate("td", "mouseout", new Function() {
						public boolean f(Event e) {
							boolean out = TableEvent.headerCellMouseOut(e);
							if (getDTable().config().isEnabledTooltip() && out) {
								tooltip.hide();
							}
							return out;
						}
					});
		} else if (!enable && frozenCellMouseOverEventHandler != null) {
			frozenCellMouseOverEventHandler.undelegate("td", "mouseover")
					.undelegate("td", "mouseout");
			frozenCellMouseOverEventHandler = null;
		}
	}

	private void activateFrozenCellClickEventHandler(boolean enable) {
		if (enable) {
			activateFrozenCellClickEventHandler(false);
			frozenCellClickEventHandler = $(
					"table#dt-frozen-header-table" + getDTable().getId())
					.delegate("td", "click", new Function() {
						public void f(Element e) {
							if (getDTable().config()
									.isEnabledSortedFrozenTable()
									&& getDTable().config().isEnabledTooltip()) {
								tooltip.hide();
							}
							TableEvent.headerClick(getDTable(), $(e));
						}
					});
		} else if (!enable && frozenCellClickEventHandler != null) {
			frozenCellClickEventHandler.undelegate("td", "click");
			frozenCellClickEventHandler = null;
		}
	}

	private void activateFrozenCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			activateFrozenCellContextMenuEventHandler(false);
			frozenCellContextMenuEventHandler = $(
					"table#dt-frozen-header-table" + getDTable().getId())
					.delegate("td", "contextmenu", new Function() {
						public boolean f(Event e) {
							if (getDTable().config().isEnabledTooltip()) {
								int column = Integer.parseInt($(e).id().split(
										":")[2]);
								if (getDTable().config()
										.isEnabledFilteredFrozenTable()
										&& getDTable().config()
												.isEnabledSortedFrozenTable()
										&& getDTable().config()
												.isColumnHideEnabled(column)
										&& getDTable().config()
												.isEnabledUnfreeze(column)) {
									tooltip.hide();
								}
							}
							if (getDTable().config().isEnabledTooltip()) {
								tooltip.hide();
							}
							return TableEvent.contextMenu(getDTable(), e);
						}
					});
		} else if (!enable && frozenCellContextMenuEventHandler != null) {
			frozenCellContextMenuEventHandler.undelegate("td", "contextmenu");
			frozenCellContextMenuEventHandler = null;
		}
	}

	private void activateCellMouseOverEventHandler(boolean enable) {
		if (enable) {
			activateCellMouseOverEventHandler(false);
			cellMouseOverEventHandler = $(
					"table#dt-header-table" + getDTable().getId()).delegate(
					"td", "mouseover", new Function() {
						public boolean f(Event e) {
							TableEvent.cellMouseOver(getDTable(), $(e), null);
							if (getDTable().config().isEnabledTooltip()
									&& !getDTable().menu().isShowing()
									&& "0".equals($(e).id().split(":")[1])) {
								int column = Integer.parseInt($(e).id().split(
										":")[2]);
								tooltip.setTo(e, getTooltip(column),
										getDTable().config()
												.isEnabledWordWrapInTooltip(),
										getDTable().config()
												.getTooltipMinimumWidth(),
										getDTable().config()
												.getTooltipMaximumWidth(),
										getDTable().config()
												.getTooltipPosition());
								tooltip.show();
							}
							return true;
						}
					}).delegate("td", "mouseout", new Function() {
				public boolean f(Event e) {
					boolean out = TableEvent.headerCellMouseOut(e);
					if (getDTable().config().isEnabledTooltip() && out) {
						tooltip.hide();
					}
					return out;
				}
			});
		} else if (!enable && cellMouseOverEventHandler != null) {
			cellMouseOverEventHandler.undelegate("td", "mouseover").undelegate(
					"td", "mouseout");
			cellMouseOverEventHandler = null;
		}
	}

	private void activateCellClickEventHandler(boolean enable) {
		if (enable) {
			activateCellClickEventHandler(false);
			cellClickEventHandler = $(
					"table#dt-header-table" + getDTable().getId()).delegate(
					"td", "click", new Function() {
						public void f(Element e) {
							if (getDTable().config().isEnabledSortedTable()
									&& getDTable().config().isEnabledTooltip()) {
								tooltip.hide();
							}
							TableEvent.headerClick(getDTable(), $(e));
						}
					});
		} else if (!enable && cellClickEventHandler != null) {
			cellClickEventHandler.undelegate("td", "click");
			cellClickEventHandler = null;
		}
	}

	private void activateCellContextMenuEventHandler(boolean enable) {
		if (enable) {
			activateCellContextMenuEventHandler(false);
			cellContextMenuEventHandler = $(
					"table#dt-header-table" + getDTable().getId()).delegate(
					"td", "contextmenu", new Function() {
						public boolean f(Event e) {
							if (getDTable().config().isEnabledTooltip()) {
								int column = Integer.parseInt($(e).id().split(
										":")[2]);
								if (getDTable().config()
										.isEnabledFilteredTable()
										&& getDTable().config()
												.isEnabledSortedTable()
										&& getDTable().config()
												.isColumnHideEnabled(column)
										&& getDTable().config()
												.isEnabledFreeze(column)) {
									tooltip.hide();
								}
							}
							return TableEvent.contextMenu(getDTable(), e);
						}
					});
		} else if (!enable && cellContextMenuEventHandler != null) {
			cellContextMenuEventHandler.undelegate("td", "contextmenu");
			cellContextMenuEventHandler = null;
		}
	}

	@Override
	public void setIds() {
		if (!headerId.endsWith("" + getDTable().getId())) {
			frozenHeaderId += getDTable().getId();
			headerId += getDTable().getId();
		}
	}

	public void readUIDL(UIDL uidl) {
		if (uidl.hasAttribute("columnsHeader")
				&& uidl.hasAttribute("visibleHeaderColumns")
				&& uidl.hasAttribute("headerColumnsAlignment")) {
			show = true;
			setColumns(uidl.getIntArrayAttribute("visibleHeaderColumns"));
			setHeaders(uidl.getStringArrayAttribute("columnsHeader"));
			setColumnsAlignment(uidl
					.getIntArrayAttribute("headerColumnsAlignment"));

			if (getDTable().config().isEnabledTooltip()
					&& uidl.hasAttribute("columnsTooltip")) {
				setTooltips(uidl.getStringArrayAttribute("columnsTooltip"));
			}

			if (uidl.hasAttribute("headerColumnsWidth")) {
				setColumnsWidth(uidl.getIntArrayAttribute("headerColumnsWidth"));
			} else {
				setColumnsWidth(getDTable().getColumnsWidth());
			}
		} else {
			show = false;
		}
	}

	public void set100percentWidth(boolean is100percentWidth) {
		$(headerId).css("width", is100percentWidth ? "100%" : "0");
	}

	@Override
	public void load() {
		setIds();
		enableEventsHandler();
	}

	@Override
	public void unload() {
		disableEventsHandler();
	}

	public void show() {
		if (show) {
			emptyHeader();
			fillHeader();

			// finalize
			removeVerticalLines();
			removeRightBorderFromLastColumn(headerId);
			setFrozenHeader();
		}
	}

	@Override
	public void enableEventsHandler() {
		activateCellClickEventHandler(getDTable().config()
				.isEnabledSortedTable());
		activateCellContextMenuEventHandler(true);
		activateCellMouseOverEventHandler(true);

		if (getDTable().config().isEnabledFrozenTable()) {
			activateFrozenCellClickEventHandler(getDTable().config()
					.isEnabledSortedFrozenTable());
			activateFrozenCellContextMenuEventHandler(true);
			activateFrozenCellMouseOverEventHandler(true);
		}
	}

	@Override
	public void disableEventsHandler() {
		removeExpandedCells();
		activateFrozenCellMouseOverEventHandler(false);
		activateFrozenCellClickEventHandler(false);
		activateFrozenCellContextMenuEventHandler(false);
		activateCellMouseOverEventHandler(false);
		activateCellClickEventHandler(false);
		activateCellContextMenuEventHandler(false);
	}

}
