/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.exporter;

import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWord;
import org.dussan.vaadin.dtable.windows.exporter.base.Exporter;
import org.dussan.vaadin.dtable.windows.exporter.base.ExportingType;

public class CsvExporter extends Exporter {

	private static final long serialVersionUID = -7064150607674977016L;
	private FileOutputStream fos = null;
	private OutputStreamWriter osw = null;
	private BufferedWriter bw = null;

	public CsvExporter(DTable dTable) {
		super(dTable);
		setCaption(getDTable().getTranslation().get(
				getDTable().getTableConfig().getLocale(),
				TranslatedWord.EXPORTING_DATA_TO_CSV_FILE));
		setExportingType(ExportingType.CSV);
	}

	private String createHeader(String name) {
		String delimiter = getDTable().getTableExporter()
				.getCsvColumnDelimiter();
		List<Integer> columns = new ArrayList<Integer>(getDTable()
				.getTableExporter().getColumnDefinition(name).keySet());
		Collections.sort(columns);

		StringBuilder header = new StringBuilder();
		for (int column : columns) {
			header.append(delimiter + "\"");
			header.append(getDTable().getTableExporter()
					.getColumnDefinitionHeader(name, column));
			header.append("\"");
		}
		return header.substring(delimiter.length());
	}

	private String row2csv(Object[] rowData) {
		String delimiter = getDTable().getTableExporter()
				.getCsvColumnDelimiter();
		StringBuilder row = new StringBuilder();
		for (Object data : rowData) {
			row.append(delimiter + "\"");
			row.append(data == null ? "" : data);
			row.append("\"");
		}
		return row.substring(delimiter.length());
	}

	@Override
	protected void export() {
		try {
			float completeStep = 0f;
			float step = 1f / getDTable().getTableExporter()
					.getColumnsDefinitionName().length;
			setExportingStatus(EXPORTING_BEGIN);
			for (String name : getDTable().getTableExporter()
					.getColumnsDefinitionName()) {
				if (openFile(name) && prepareExportedData(name)
						&& canContinue()) {
					writeToFile(name, completeStep, step);
					closeFile();
					completeStep += step;
				} else {
					setExportingStatus(EXPORTING_ERROR);
				}
				// check status
				if (!canContinue()) {
					closeFile();
					if (getExportingStatus() == EXPORTING_STOP) {
						setExportingStatus(EXPORTING_STOPPED);
					}
					return;
				}
			}
			setExportingStatus(EXPORTING_COMPLETE);
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	@Override
	protected boolean openFile(String name) {
		try {
			fos = new FileOutputStream(getTempFile(name));
			osw = new OutputStreamWriter(fos, getCharset());
			bw = new BufferedWriter(osw);
			return true;
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
			return false;
		}
	}

	@Override
	protected void writeToFile(String name, float completeStep, float step) {
		int row=0,rows=0;
		try {
			long allRows = getAllRows(name);
			int allColumns = getAllColumns(name);
			int exportedStep = allRows < EXPORTING_NORMAL_STEP ? (int) allRows
					: EXPORTING_NORMAL_STEP;
			StringBuilder exportedData = new StringBuilder();
			exportedData.append(createHeader(name) + "\n");
			for (int mainRow = 0; mainRow < allRows; mainRow += exportedStep) {
				rows = (int) (mainRow + exportedStep <= allRows ? exportedStep
						: allRows - mainRow);
				Object[] rowsData = getData(name, mainRow, rows);
				for (row = 0; row < rows; row++) {
					exportedData.append(row2csv(Arrays.copyOfRange(rowsData,
							row * allColumns, (row + 1) * allColumns)) + "\n");
					// exports was stopped
					if (!canContinue()) {
						setExportingStatus(EXPORTING_STOPPED);
						return;
					}
				}
				setExportingStatus((step * mainRow / allRows) + completeStep);
				bw.write(exportedData.toString());
				exportedData = new StringBuilder();
			}
		} catch (Exception e) {
			row+=0;
			rows+=0;
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	@Override
	protected void closeFile() {
		try {
			bw.close();
			osw.close();
			fos.close();
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

}
