/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.exporter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWord;
import org.dussan.vaadin.dtable.windows.exporter.base.Exporter;
import org.dussan.vaadin.dtable.windows.exporter.base.ExportingColumnType;
import org.dussan.vaadin.dtable.windows.exporter.base.ExportingType;

import com.healthmarketscience.jackcess.ColumnBuilder;
import com.healthmarketscience.jackcess.Database;
import com.healthmarketscience.jackcess.JetFormat;
import com.healthmarketscience.jackcess.TableBuilder;

public class MdbExporter extends Exporter {

	private static final long serialVersionUID = -7064150607674977016L;
	private Database database = null;
	private List<String> tablesName = new ArrayList<String>();
	private Map<Integer, Integer> columnsStringValueMaxLength = new HashMap<Integer, Integer>();
	private int columnStep = 255;

	public MdbExporter(DTable dTable) {
		super(dTable);
		setCaption(getDTable().getTranslation().get(
				getDTable().getTableConfig().getLocale(),
				TranslatedWord.EXPORTING_DATA_TO_MS_ACCESS_FILE));
		setExportingType(ExportingType.MS_ACCESS);
	}

	private List<Object> checkStringValuesLength(int allColumns, int allRows,
			List<Object> data) {
		if (!columnsStringValueMaxLength.isEmpty()) {
			try {
				for (int row = 0; row < allRows; row++) {
					List<Object> values = data.subList(allColumns * row,
							allColumns * (row + 1));
					for (int column : columnsStringValueMaxLength.keySet()) {
						int maxLength = columnsStringValueMaxLength.get(column);
						String value = values.get(column).toString();
						if (value != null && value.length() > maxLength) {
							data.set(allColumns * row + column,
									value.substring(0, maxLength));
						}
					}
				}
			} catch (Exception e) {
			}
		}
		return data;
	}

	/*
	 * Object[] definition: 0: column header name, 1: formatting, 2: exported
	 * column type, 3: length or precision
	 */
	private void createTablesHeader(String name) {
		try {
			tablesName.clear();
			columnsStringValueMaxLength.clear();

			int columnsDefinitionSize = getDTable().getTableExporter()
					.getColumnDefinition(name).size();
			List<Integer> columns = new ArrayList<Integer>(getDTable()
					.getTableExporter().getColumnDefinition(name).keySet());
			Collections.sort(columns);

			for (int table = 0; table <= (columnsDefinitionSize / columnStep); table++) {
				String tableName = name
						+ (columnsDefinitionSize <= columnStep ? ""
								: (table + 1));
				TableBuilder tableBuilder = new TableBuilder(tableName);
				for (int column = columnStep * table; column < (columnStep * (table + 1))
						&& column < columnsDefinitionSize; column++) {
					String header = getDTable().getTableExporter()
							.getColumnDefinitionHeader(name,
									columns.get(column));
					ExportingColumnType columnExportedType = getDTable()
							.getTableExporter()
							.getColumnDefinitionExportingType(name,
									columns.get(column));
					Integer lengthOrPrecision = getDTable().getTableExporter()
							.getColumnDefinitionLengthOrPrecision(name,
									columns.get(column));

					ColumnBuilder tableColumn = new ColumnBuilder(header,
							columnExportedType.getType());
					if (lengthOrPrecision != null) {
						switch (columnExportedType.getType()) {
						case DOUBLE:
						case FLOAT:
							tableColumn.setPrecision(lengthOrPrecision);
							break;
						case TEXT:
							lengthOrPrecision = lengthOrPrecision <= JetFormat.TEXT_FIELD_MAX_LENGTH ? lengthOrPrecision
									: JetFormat.TEXT_FIELD_MAX_LENGTH;
							tableColumn.setLengthInUnits(lengthOrPrecision);
							columnsStringValueMaxLength.put(column,
									lengthOrPrecision);
							break;
						default:
							break;
						}
					} else if (ExportingColumnType.TEXT
							.equals(columnExportedType)
							&& lengthOrPrecision == null) {
						tableColumn.setType(ExportingColumnType.MEMO.getType());
					}
					tableBuilder.addColumn(tableColumn.toColumn());
				}
				tableBuilder.toTable(database);
				tablesName.add(tableName);
			}
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	@Override
	protected void export() {
		try {
			float completeStep = 0f;
			float step = 1f / getDTable().getTableExporter()
					.getColumnsDefinitionName().length;
			setExportingStatus(EXPORTING_BEGIN);
			if (openFile(getDTable().getTableExporter().getFilename())) {
				for (String name : getDTable().getTableExporter()
						.getColumnsDefinitionName()) {
					if (prepareExportedData(name) && canContinue()) {
						writeToFile(name, completeStep, step);
						completeStep += step;
					} else {
						setExportingStatus(EXPORTING_ERROR);
					}
					// check status
					if (!canContinue()) {
						closeFile();
						if (getExportingStatus() == EXPORTING_STOP) {
							setExportingStatus(EXPORTING_STOPPED);
						}
						return;
					}
				}
			} else {
				closeFile();
				setExportingStatus(EXPORTING_ERROR);
				return;
			}
			closeFile();
			setExportingStatus(EXPORTING_COMPLETE);
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	@Override
	protected boolean openFile(String name) {
		try {
			database = Database.create(getDTable().getTableExporter()
					.getMsFileVersion().getVersion(), getTempFile(name), false);
			return true;
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
			return false;
		}
	}

	@Override
	protected void writeToFile(String name, float completeStep, float step) {
		try {
			createTablesHeader(name);
			if (getExportingStatus() != EXPORTING_ERROR) {
				long allRows = getAllRows(name);
				int allColumns = getAllColumns(name);
				if (allColumns > 0 && allRows > 0) {
					int exportedStep = allRows < EXPORTING_NORMAL_STEP ? (int) allRows
							: EXPORTING_NORMAL_STEP;

					for (int mainRow = 0; mainRow < allRows; mainRow += exportedStep) {
						int rows = (int) (mainRow + exportedStep <= allRows ? exportedStep
								: allRows - mainRow);
						List<Object[]> data = new ArrayList<Object[]>();
						List<Object> rowsData = checkStringValuesLength(
								allColumns, exportedStep,
								Arrays.asList(getData(name, mainRow, rows)));

						for (int table = 0; table < tablesName.size(); table++) {
							int fromColumn = columnStep * table;
							int toColumn = columnStep * (table + 1);
							toColumn = Math.min(toColumn, allColumns);

							data.clear();
							for (int row = 0; row < rows; row++) {
								data.add(rowsData.subList(
										allColumns * row + fromColumn,
										allColumns * row + toColumn).toArray());
								// exports was stopped
								if (!canContinue()) {
									setExportingStatus(EXPORTING_STOPPED);
									return;
								}
							}
							database.getTable(tablesName.get(table)).addRows(
									data);
						}
						setExportingStatus((step * mainRow / allRows)
								+ completeStep);
					}
				} else {
					setExportingStatus(
							EXPORTING_ERROR,
							getDTable().getTranslation().get(
									getDTable().getTableConfig().getLocale(),
									TranslatedWord.NO_DATA_FOR_EXPORT));
				}
			}
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

	@Override
	protected void closeFile() {
		try {
			database.close();
		} catch (Exception e) {
			setExportingStatus(EXPORTING_ERROR, e.getLocalizedMessage());
		}
	}

}
