/**
 * Copyright (C) 2012  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.windows.freeze;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang3.ArrayUtils;
import org.dussan.vaadin.dtable.DTable;
import org.dussan.vaadin.dtable.translations.TranslatedWord;

import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.ListSelect;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

public class FreezeUnfreezeColumns extends Window {

	private static final long serialVersionUID = 3771051434575477411L;
	private DTable dTable = null;
	private boolean isAttached = false;
	private String hiddenAddition = null;

	private StringBuilder jsDisabledUnfreezeColumns = null;
	private StringBuilder jsDisabledFreezeColumns = null;
	private ListSelect unfrozenColumns = null;
	private ListSelect frozenColumns = null;
	private Button closeButton = null;
	private Button okButton = null;
	private Button freezeButton = null;
	private Button unfreezeButton = null;

	public FreezeUnfreezeColumns(DTable dTable) {
		this.dTable = dTable;
	}

	private DTable getDTable() {
		return dTable;
	}

	private void closeWindow() {
		setVisible(false);
		getParent().removeWindow(this);
	}

	private void setVisibleAndHiddenColumns() {
		unfrozenColumns.removeAllItems();
		if (!getDTable().getTableProperties().isNoHiddenColumn()) {
			for (String column : getDTable().getTableProperties()
					.getHiddenColumnsString()) {
				int col = Integer.parseInt(column.substring(1));
				if (!getDTable().getTableProperties().isHiddenColumnFrozen(col)) {
					unfrozenColumns.addItem(getDTable().getTableProperties()
							.getColumnHeader(col) + hiddenAddition);
				}
			}
		}

		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			if (!getDTable().getTableProperties().isFrozenColumn(column)
					&& !getDTable().getTableProperties().isHiddenColumn(column)) {
				unfrozenColumns.addItem(getDTable().getTableProperties()
						.getColumnHeader(column));
			}
		}

		frozenColumns.removeAllItems();
		if (!getDTable().getTableProperties().isNoFrozenColumn()) {
			if (!getDTable().getTableProperties().isNoHiddenColumn()) {
				for (String column : getDTable().getTableProperties()
						.getHiddenColumnsString()) {
					int col = Integer.parseInt(column.substring(1));
					if (getDTable().getTableProperties().isHiddenColumnFrozen(
							col)) {
						frozenColumns.addItem(getDTable().getTableProperties()
								.getColumnHeader(col) + hiddenAddition);
					}
				}
			}

			for (int column : getDTable().getTableProperties()
					.getFrozenColumns()) {
				frozenColumns.addItem(getDTable().getTableProperties()
						.getColumnHeader(column));
			}
		}

		jsDisabledUnfreezeColumns = new StringBuilder();
		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			String header = getDTable().getTableProperties().getColumnHeader(
					column);
			if (!getDTable().getTableConfig().canColumnUnfreeze(column)) {
				jsDisabledUnfreezeColumns.append(",'" + header + hiddenAddition
						+ "'");
				jsDisabledUnfreezeColumns.append(",'" + header + "'");
			}
		}
		if (jsDisabledUnfreezeColumns.length() > 0) {
			jsDisabledUnfreezeColumns = new StringBuilder("["
					+ jsDisabledUnfreezeColumns.substring(1) + "]");
		}

		jsDisabledFreezeColumns = new StringBuilder();
		for (int column = 0; column < getDTable().getTableProperties()
				.getAllColumns(); column++) {
			String header = getDTable().getTableProperties().getColumnHeader(
					column);
			if (!getDTable().getTableConfig().canColumnFreeze(column)) {
				jsDisabledFreezeColumns.append(",'" + header + hiddenAddition
						+ "'");
				jsDisabledFreezeColumns.append(",'" + header + "'");
			}
		}
		if (jsDisabledFreezeColumns.length() > 0) {
			jsDisabledFreezeColumns = new StringBuilder("["
					+ jsDisabledFreezeColumns.substring(1) + "]");
		}
	}

	private boolean isAtLeastOneColumnUnhidden(Collection<?> ids) {
		int unhiddenColumns = 0;
		for (Object id : ids) {
			String idString = id.toString();
			if (!idString.endsWith(hiddenAddition)) {
				unhiddenColumns += 1;
			}
		}
		return (unhiddenColumns > 0);
	}

	public void refreshDisabledColumns(String jsDisabledColumns, String id) {
		if (!jsDisabledColumns.isEmpty()) {
			StringBuilder js = new StringBuilder();
			js.append("setTimeout(function(){var s=");
			js.append(jsDisabledColumns);
			js.append(";var i=document.getElementById('");
			js.append(id);
			js.append("').children[0].options.length;for(var iii=0;iii<s.length;iii++){for(var ii=0;ii<i;ii++){if(s[iii]==document.getElementById('");
			js.append(id);
			js.append("').children[0].options[ii].text){document.getElementById('");
			js.append(id);
			js.append("').children[0].options[ii].disabled=true;}}}},150);");
			getApplication().getMainWindow().executeJavaScript(js.toString());
		}
	}

	@Override
	public void attach() {
		if (!isAttached) {
			hiddenAddition = " ("
					+ getDTable().getTranslation().get(
							getDTable().getTableConfig().getLocale(),
							TranslatedWord.HIDDEN) + ")";

			VerticalLayout windowLayout = (VerticalLayout) getContent();
			windowLayout.addStyleName("v-dtable-dialog");
			windowLayout.setMargin(false);
			windowLayout.setSpacing(false);

			setCaption(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.FREEZE_UNFREEZE_COLUMNS));
			center();
			setClosable(false);
			setDraggable(false);
			setResizable(false);
			setModal(true);
			setWidth(600, UNITS_PIXELS);

			HorizontalLayout layout = new HorizontalLayout();
			layout.addStyleName("context");
			layout.setWidth(100, UNITS_PERCENTAGE);
			layout.setMargin(true);
			layout.setSpacing(true);
			windowLayout.addComponent(layout);

			unfrozenColumns = new ListSelect(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.UNFROZEN_COLUMNS));
			unfrozenColumns.setDebugId("unfrozenColumns");
			unfrozenColumns.setWidth(100, UNITS_PERCENTAGE);
			unfrozenColumns.setNullSelectionAllowed(true);
			unfrozenColumns.setMultiSelect(true);
			unfrozenColumns.setImmediate(true);
			unfrozenColumns.setRows(25);
			layout.addComponent(unfrozenColumns);

			VerticalLayout actionLayout = new VerticalLayout();
			actionLayout.setCaption("");
			actionLayout.setSizeUndefined();
			actionLayout.setMargin(false);
			actionLayout.setSpacing(false);
			layout.addComponent(actionLayout);

			frozenColumns = new ListSelect(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.FROZEN_COLUMNS));
			frozenColumns.setDebugId("frozenColumns");
			frozenColumns.setWidth(100, UNITS_PERCENTAGE);
			frozenColumns.setNullSelectionAllowed(true);
			frozenColumns.setMultiSelect(true);
			frozenColumns.setImmediate(true);
			frozenColumns.setRows(25);
			layout.addComponent(frozenColumns);
			layout.setExpandRatio(unfrozenColumns, 1.0f);
			layout.setExpandRatio(frozenColumns, 1.0f);
			setVisibleAndHiddenColumns();

			freezeButton = new Button(">>", new ClickListener() {
				private static final long serialVersionUID = 4097366270472216209L;

				@Override
				public void buttonClick(ClickEvent event) {
					Object values = unfrozenColumns.getValue();
					if (values != null) {
						for (Object value : (Set<?>) values) {
							unfrozenColumns.removeItem(value);
							frozenColumns.addItem(value);
						}
					}
					unfrozenColumns.select(null);
					refreshDisabledColumns(
							jsDisabledUnfreezeColumns.toString(),
							frozenColumns.getDebugId());
					refreshDisabledColumns(jsDisabledFreezeColumns.toString(),
							unfrozenColumns.getDebugId());
				}
			});
			actionLayout.addComponent(freezeButton);

			unfreezeButton = new Button("<<", new ClickListener() {
				private static final long serialVersionUID = -6384761875661773041L;

				@Override
				public void buttonClick(ClickEvent event) {
					Object values = frozenColumns.getValue();
					if (values != null) {
						for (Object value : (Set<?>) values) {
							frozenColumns.removeItem(value);
							unfrozenColumns.addItem(value);
						}
					}
					frozenColumns.select(null);
					refreshDisabledColumns(
							jsDisabledUnfreezeColumns.toString(),
							frozenColumns.getDebugId());
					refreshDisabledColumns(jsDisabledFreezeColumns.toString(),
							unfrozenColumns.getDebugId());
				}
			});
			actionLayout.addComponent(unfreezeButton);

			HorizontalLayout space1 = new HorizontalLayout();
			space1.setWidth(100, UNITS_PERCENTAGE);
			space1.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space1);
			windowLayout.setComponentAlignment(space1, Alignment.BOTTOM_CENTER);

			HorizontalLayout buttonLayout = new HorizontalLayout();
			buttonLayout.setWidth(100, UNITS_PERCENTAGE);
			buttonLayout.setMargin(false);
			buttonLayout.setSpacing(true);
			windowLayout.addComponent(buttonLayout);
			windowLayout.setComponentAlignment(buttonLayout,
					Alignment.BOTTOM_CENTER);

			HorizontalLayout space2 = new HorizontalLayout();
			space2.setWidth(100, UNITS_PERCENTAGE);
			space2.setHeight(6, UNITS_PIXELS);
			windowLayout.addComponent(space2);
			windowLayout.setComponentAlignment(space2, Alignment.BOTTOM_CENTER);

			okButton = new Button(getDTable().getTranslation()
					.get(getDTable().getTableConfig().getLocale(),
							TranslatedWord.OK), new ClickListener() {
				private static final long serialVersionUID = -5941114122010602045L;

				@Override
				public void buttonClick(ClickEvent event) {
					List<Integer> columns = new ArrayList<Integer>();
					if (frozenColumns.getItemIds() != null) {
						for (Object id : (Collection<?>) frozenColumns
								.getItemIds()) {
							String idString = id.toString();
							if (idString.endsWith(hiddenAddition)) {
								idString = idString.substring(0,
										idString.lastIndexOf(hiddenAddition));
							}
							columns.add(getDTable().getTableProperties()
									.getColumnHeaderIndex(idString));
						}
					}

					if (!isAtLeastOneColumnUnhidden(unfrozenColumns
							.getItemIds())) {
						getApplication()
								.getMainWindow()
								.showNotification(
										getDTable()
												.getTranslation()
												.get(getDTable()
														.getTableConfig()
														.getLocale(),
														TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_UNFROZEN_AND_UNHIDDEN),
										Notification.TYPE_ERROR_MESSAGE);
					} else if (!getDTable().getTableConfig()
							.isEnabledEmptyFrozenTable()
							&& !isAtLeastOneColumnUnhidden(frozenColumns
									.getItemIds())) {
						getApplication()
								.getMainWindow()
								.showNotification(
										getDTable()
												.getTranslation()
												.get(getDTable()
														.getTableConfig()
														.getLocale(),
														TranslatedWord.AT_LEAST_ONE_COLUMN_HAVE_TO_BE_FROZEN_AND_UNHIDDEN),
										Notification.TYPE_ERROR_MESSAGE);
					} else {
						getDTable().getTableProperties().resetFrozenColumns();
						if (!columns.isEmpty()) {
							getDTable().getTableProperties().setFrozenColumns(
									ArrayUtils.toPrimitive(columns
											.toArray(new Integer[] {})));
						}
						closeWindow();
						getDTable().refreshVisibleData(true);
					}
				}
			});
			buttonLayout.addComponent(okButton);
			buttonLayout
					.setComponentAlignment(okButton, Alignment.MIDDLE_RIGHT);

			closeButton = new Button(getDTable().getTranslation().get(
					getDTable().getTableConfig().getLocale(),
					TranslatedWord.CLOSE), new ClickListener() {
				private static final long serialVersionUID = 5390169876653829863L;

				@Override
				public void buttonClick(ClickEvent event) {
					closeWindow();
				}
			});
			buttonLayout.addComponent(closeButton);
			buttonLayout.setComponentAlignment(closeButton,
					Alignment.MIDDLE_LEFT);

			refreshDisabledColumns(jsDisabledUnfreezeColumns.toString(),
					frozenColumns.getDebugId());
			refreshDisabledColumns(jsDisabledFreezeColumns.toString(),
					unfrozenColumns.getDebugId());
			isAttached = true;
		}
		super.attach();
	}

	@Override
	public void detach() {
		if (isAttached) {
			removeAllComponents();
			isAttached = false;
		}
		super.detach();
	}

}
